<?php

namespace DHLParcel\Shipping\Model\Service;

use DHLParcel\Shipping\Model\Core\SingletonAbstract;
use DHLParcel\Shipping\Model\Core\TranslateTrait;
use DHLParcel\Shipping\Model\Data\Api\Response\Capability\Option;
use Configuration;
use DHLParcel\Shipping\Model\Core\Settings;

class BulkProfileService extends SingletonAbstract
{
    use TranslateTrait;

    const LANG_GROUP_BUSINESS = 'business';
    const LANG_GROUP_DELIVERY = 'delivery';
    const LANG_GROUP_OPTIONS = 'options';

    /** Return allowed bulk options, ignore everything else */
    public function options()
    {
        return [
            // Delivery methods for preset settings for carriers
            Option::KEY_DOOR,
            Option::KEY_BP,
            // Note: ServicePoint selection will always take precedent.
            // Service methods for preset settings for carriers
            Option::KEY_SDD,
            Option::KEY_EA,
            Option::KEY_HANDT,
            Option::KEY_EVE,
            Option::KEY_NBB,
            Option::KEY_AGE_CHECK,
            Option::KEY_S,
            Option::KEY_EXP,
        ];
    }

    public function getTranslationGroups()
    {
        return [
            self::LANG_GROUP_BUSINESS,
            self::LANG_GROUP_DELIVERY,
            self::LANG_GROUP_OPTIONS
        ];
    }

    public function getTranslations()
    {
        $optionService = OptionService::instance();
        return [
            self::LANG_GROUP_BUSINESS => [
                ''         => 'Default shipment type',
                'private'  => 'Send to consumer',
                'business' => 'Send to business',
            ],
            self::LANG_GROUP_DELIVERY => [
                ''             => 'Default delivery method',
                Option::KEY_BP => $optionService->getLabel(Option::KEY_BP),
            ],
            self::LANG_GROUP_OPTIONS  => [
                Option::KEY_SDD       => $optionService->getLabel(Option::KEY_SDD),
                Option::KEY_EA        => $optionService->getLabel(Option::KEY_EA),
                Option::KEY_HANDT     => $optionService->getLabel(Option::KEY_HANDT),
                Option::KEY_EVE       => $optionService->getLabel(Option::KEY_EVE),
                Option::KEY_NBB       => $optionService->getLabel(Option::KEY_NBB),
                Option::KEY_AGE_CHECK => $optionService->getLabel(Option::KEY_AGE_CHECK),
                Option::KEY_INS       => $optionService->getLabel(Option::KEY_INS),
                Option::KEY_S         => $optionService->getLabel(Option::KEY_S),
                Option::KEY_EXP       => $optionService->getLabel(Option::KEY_EXP),
            ],
        ];
    }

    public function translate($group, $key)
    {
        return isset($this->getTranslations()[$group][$key]) ? $this->getTranslations()[$group][$key] : '???';
    }

    public function load($code)
    {
        $profiles = $this->getProfiles();
        foreach ($profiles as $profile) {
            if ($profile->code == $code) {
                return $profile;
            }
        }
        return null;
    }

    public function getDropdownData()
    {
        $profiles = $this->getProfiles();
        $dropdown = [];

        // Menu: "create label"
        foreach ($profiles as $profile) {
            $subtext = $this->translate(self::LANG_GROUP_BUSINESS, $profile->business);
            $subtext .= ",\n" . $this->translate(self::LANG_GROUP_DELIVERY, $profile->delivery);
            foreach ($profile->options as $option) {
                $subtext .= ",\n" . $this->translate(self::LANG_GROUP_OPTIONS, $option);
                if (in_array($option, $profile->required_options)) {
                    $subtext .= '*';
                }
            }

            $subcode = $profile->delivery;
            foreach ($profile->options as $option) {
                $subcode .= ($subcode ? ', ' : '') . $option;
                if (in_array($option, $profile->required_options)) {
                    $subtext .= '*';
                }
            }
            $subcode = $this->translate(self::LANG_GROUP_BUSINESS, $profile->business) . ' [' . $subcode  . ']';

            $dropdown[] = [
                'icon' => 'icon-plane',
                'text' => 'Create ' . strtoupper($profile->size) . ' labels [' . strtoupper($profile->country). ']',
                'subtext' => $subtext,
                'subcode' => $subcode,
                'type' => 'create',
                'code' => $profile->code,
            ];
        }

        // Menu: "download label"
        $dropdown[] = [
            'icon' => 'icon-download',
            'text' => 'Download labels',
            'type' => 'download',
            'code' => 'download',
        ];

        // TODO Menu: "print label"
//        $dropdown[] = [
//            'icon' => 'icon-print',
//            'text' => 'Print labels',
//            'type' => 'print',
//            'code' => 'print',
//        ];

        return $dropdown;
    }

    public function getProfiles()
    {

        $json = Configuration::get(Settings::BULK_PROFILES_DATA);
        if (!$json) {
            return [];
        }

        $data = json_decode($json);
        if (json_last_error() !== JSON_ERROR_NONE) {
            return [];
        }

        if (!is_array($data)) {
            return [];
        }

        foreach ($data as $row) {
            if (!property_exists($row, 'code') ||
                !property_exists($row, 'country') ||
                !property_exists($row, 'business') ||
                !property_exists($row, 'delivery') ||
                !property_exists($row, 'options') ||
                !is_array($row->options) ||
                !property_exists($row, 'required_options') ||
                !is_array($row->required_options) ||
                !property_exists($row, 'size')
            ) {
                return [];
            }
        }

        return $data;
    }
}
